"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addAbortSignal = exports.ErrorBoundary = void 0;
const internal_1 = require("./internal");
const domUtils_1 = require("./domUtils");
exports.ErrorBoundary = (0, internal_1.internalComponent)((props) => async (document, thisArg) => {
    const abortController = new AbortController();
    // Connect to parent AbortSignal
    const abort = () => {
        !abortController.signal.aborted && abortController.abort();
    };
    thisArg.abortSignal.addEventListener("abort", abort);
    try {
        const children = await props.render();
        return (0, domUtils_1.toDom)(document, children, addAbortSignal(thisArg, abortController));
    }
    catch (error) {
        abort();
        if (props.accept && !props.accept(error)) {
            throw error;
        }
        const children = await props.fallback({ error });
        return (0, domUtils_1.toDom)(document, children, thisArg);
    }
});
// Fixme: Experimental idea to support cancel parallel requests when one of the requests fails within an error-boundary.
// Fixme: The only other alternative to support early bailout like above would be to use generator functions.
// Generators seem like the perfect fit, but they are not widely used. Async/await is more approachable.
// If generators will be used, keep async functions or replace them entirely with generators?
function addAbortSignal(thisArg, abortController) {
    const abortSignal = abortController.signal;
    return {
        ...thisArg,
        abortSignal,
        withAbortSignal(promise) {
            return new Promise((resolve, reject) => {
                const bail = () => reject();
                abortSignal.addEventListener("abort", bail);
                promise
                    .then((value) => {
                    if (!abortSignal.aborted) {
                        abortSignal.removeEventListener("abort", bail);
                        resolve(value);
                    }
                })
                    .catch((error) => {
                    if (!abortSignal.aborted) {
                        abortSignal.removeEventListener("abort", bail);
                        abortController.abort();
                        reject(error);
                    }
                });
            });
        },
    };
}
exports.addAbortSignal = addAbortSignal;
//# sourceMappingURL=ErrorBoundary.js.map